@@ -33,14 +33,11 @@ my $n_tree = $op->new( {
 	operands => [$exp, $a],
 } );
 
-#print "$n_tree\n";
 print $n_tree->to_string('prefix') . " = " . $n_tree->value() . "\n\n";
 
 print "Now, we apply the derivative to the term: (infix)\n";
 my $derived = $n_tree->apply_derivatives();
 
-print "$derived\n";
-
 print "$derived = " . $derived->value() . "\n\n";
 
 print "Finally, we simplify the derived term as much as possible:\n";
